/**
 * This class manages an action that's "time buffered".
 * A client can "request" an action to be done *later*.
 * The callback will be triggered when the time's up, but if the client
 * requests the action again, the original timer will cancel.
 */
class Mainspring {
    constructor(options) {
        this.options = options;
        this._hasActiveTimer = false;
    }
    arm() {
        if (this._hasActiveTimer) {
            this.cancel();
        }
        this._hasActiveTimer = true;
        this._timer = setTimeout(() => {
            this._handleTimerExpiry();
        }, this.options.delay ?? 1000);
    }
    cancel() {
        // sanity check
        if (!this._hasActiveTimer) {
            return;
        }
        clearTimeout(this._timer);
        this._hasActiveTimer = false;
    }
    _handleTimerExpiry() {
        this._hasActiveTimer = false;
        this.options.callback();
    }
}

/**
 * Debounced-mutation-observer
 *
 * ----
 *
 * @author  rileyvel
 * @version 0
 * @licence blessing
 *
 */
class DebouncedMutationObserver {
    /**
     * Creates a new debounced mutation observer.
     * NOTE - monitoring does not start until you call the observe method
     * @param callback
     * @param delay in ms
     */
    constructor(callback, delay) {
        this.rawCallback = () => {
            this.mainspring.arm();
        };
        this.mainspring = new Mainspring({ callback, delay });
        this.observer = new MutationObserver(this.rawCallback);
    }
    /**
     * Observe an element. Including its childList, attributes and subtree.
     * @param element
     */
    observe(element) {
        this.observer.observe(element, {
            childList: true,
            attributes: true,
            subtree: true,
            characterData: true
        });
    }
    /**
     * Disconnect the observer
     */
    disconnect() {
        this.observer.disconnect();
    }
    /**
     * Removes all pending notifications from the MutationObserver's notification queue and
     * returns them in a new Array of MutationRecord objects.
     * Same as MutationObserver's takeRecords.
     */
    takeRecords() {
        return this.observer.takeRecords();
    }
}

export { DebouncedMutationObserver };
